declare namespace x="http://www.datadirect.com/xquery";

declare variable $planExplainDoc as document-node(element(*, xs:untyped)) external;
declare variable $resourceLocation as xs:string external;

declare function local:getResouceLocation() as xs:string
{
	if(string-length($resourceLocation) > 0) then
		let $withScheme :=
			if(starts-with($resourceLocation, "file:") or starts-with($resourceLocation, "http:")) then
				$resourceLocation
			else
				concat("file:", $resourceLocation)
		let $lastChar := substring($withScheme, string-length($withScheme), 1)
		return
			if($lastChar = "/" or $lastChar = "\") then
				$withScheme
			else
				concat($withScheme, "/")
	else
		"http://www.xquery.com/planExplain3/"
};

declare function local:getSQLStatementDescription($statement as xs:string, $isAdaptor as xs:boolean) as xs:string
{
	if($statement = 'select')
	then "Select"
	else
		if($statement = 'update' or $statement = 'delete')
		then 
			if($isAdaptor)
			then concat("Deferred ", $statement)
			else if($statement = 'update') then "Update" else "Delete"
		else
			if($statement = 'create_temp')
				then "Create temporary table"
				else
				if($statement = 'insert_temp')
					then "Insert in temporary table"
					else $statement
};

declare function local:renderConstructor($constructor as element(*, xs:untyped))
{
  let $name := if ($constructor/@name) then concat("$",$constructor/@name) else "&lt;b&gt;return&lt;/b&gt;"
  let $icon := if ($constructor/@name) then "constructor.gif" else "return.gif"
  return
  <item text="{$name}" im0="{$icon}" im1="{$icon}" im2="{$icon}">
  { 
       if(count($constructor/*[@type='input'])>0) then
           <item text="&lt;b&gt;Inputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
		   {
		       for $in at $pos in $constructor/* where $in/@type='input' order by $pos descending
		       return local:switch($in)
		   }
		   </item>
	   else
	       ""
  }
    <item text="{$constructor/x:expression/text()}" im0="expression.gif" im1="expression.gif" im2="expression.gif"/>
  </item>
};

declare function local:renderSQLOutputs($sql_statement as element(*, xs:untyped))
{
   if(count($sql_statement/*[@type="output"])>0) then
       <item text="&lt;b&gt;Outputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
       {
          for $output in $sql_statement/* where $output/@type="output"
	      return 
			  if ($output/local-name()='constructor') then
			  	  local:renderConstructor($output)
			  else 
			      <item text="${$output/@name} := {$output/text()}" im0="var.gif" im1="var.gif" im2="var.gif"/>
       }
       </item>
   else
	   ""
};

declare function local:adaptorsSection($context as element(*, xs:untyped)) {
    if(count($context/x:adaptors/x:adaptor)>0) then 
		<item text="&lt;b&gt;Adaptors&lt;b/&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif"> 
		{
			for $adaptor in $context/x:adaptors/x:adaptor
			return 
				<item text="{$adaptor/@url}" im0="adaptor.gif" im1="adaptor.gif" im2="adaptor.gif"> 
				{
				   if(count($adaptor/x:sql)>0) then
				      <item text="&lt;b&gt;Temporary Tables&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif"> 
					  {
					      for $sql_statement in $adaptor/x:sql
						  return
					          <item text="&lt;b&gt;SQL:&lt;/b&gt; {string($sql_statement)}" im0="sql.gif" im1="sql.gif" im2="sql.gif">
					          </item>
					  }
					  </item>				       
				   else
				       ""
				}
				{
				   for $statement at $pos in $adaptor/* where $statement/local-name()='update' or $statement/local-name()='create_temp' or $statement/local-name()='insert_temp' or $statement/local-name()='delete' or $statement/local-name()='select' order by $pos descending
				   return
				      <item text="&lt;b&gt;{local:getSQLStatementDescription($statement/local-name(), true())}&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif"> 
		              {
				            if(count($statement/*[local-name()="parameter"])>0) then
					            <item text="&lt;b&gt;Inputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
						        {
							      for $input in $statement/* where $input/local-name()="parameter"
								  return 
								      <item text="param {$input/@pos} := ${$input/@name}" im0="varRef.gif" im1="varRef.gif" im2="varRef.gif"/>
							    }
								</item>
							   else
								 ""
					  }
					  {
						   if(count($statement/*[local-name()="variable"])>0) then
						       <item text="&lt;b&gt;Outputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
						       {
						          for $output in $statement/* where $output/local-name()="variable"
							      return 
							          <item text="${$output/@name} := {$output/text()}" im0="var.gif" im1="var.gif" im2="var.gif"/>
						       }
						       </item>
						   else
							   ""
					  }
				      {
					      for $sql_statement in $statement/x:sql
						  return
					          <item text="&lt;b&gt;SQL:&lt;/b&gt; {string($sql_statement)}" im0="sql.gif" im1="sql.gif" im2="sql.gif">
					          </item>
			          }
					  </item>
				}
				</item>	   
		}
		</item>	
   else ""   
};


declare function local:globalVariablesSection($context as element(*, xs:untyped)) {
    if(count($context/x:plan[@type="variable declaration"])>0) then 
		<item text="&lt;b&gt;Global Variables&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif"> 
		{
			for $variable at $pos in $context/x:plan where $variable/@type="variable declaration" order by $pos descending
			return 
				<item text="${$variable/@name} {if(count($variable/@original_name)>0) then concat(' (',concat($variable/@original_name,')')) else ''}" im0="var.gif" im1="var.gif" im2="var.gif"> 
				    { if($variable/x:source) then
						local:source($variable/x:source)
					  else
					  	for $item in $variable/*  return local:switch($item)
					}
				</item>	   
		}
		</item>	
   else ""   
};

declare function local:functionsSection($context as element(*, xs:untyped)) {
    if(count($context/x:plan[@type="function declaration"])>0) then 
		<item text="&lt;b&gt;Local Functions&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif"> 
		{
			for $function in $context/x:plan where $function/@type="function declaration"
			return 
				<item text="{$function/@name}" im0="function.gif" im1="function.gif" im2="function.gif"> 
				    {
			          if($function/x:parameters) then
			            <item text="&lt;b&gt;Inputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
				        {
					      for $input in $function/x:parameters/x:parameter
						  return 
						      <item text="${$input/@name} := param {$input/@pos}" im0="var.gif" im1="var.gif" im2="var.gif"/>
					    }
						</item>
					  else
						""
				    }
				    { if($function/x:source) then
						local:source($function/x:source)
					  else
					  	for $item in $function/*  return local:switch($item)
					}
				</item>	   
		}
		</item>
	else ""
};


declare function local:mainSection($context as element(*, xs:untyped)) {
	for $main in $context/x:plan where string($main/@name)="main"
	return 
	   <item text="&lt;b&gt;Plan&lt;/b&gt;" im0="plan.gif" im1="plan.gif" im2="plan.gif">
	   {
		  for $item in $main/*  return local:switch($item)  
	   }
	   </item>	   
};


declare function local:switch($context as element(*, xs:untyped)) {
  if($context/local-name()="FLWOR")                then local:flwor($context)
  else if($context/local-name()="order_by")        then local:order_by($context)
  else if($context/local-name()="let")             then local:let($context)
  else if($context/local-name()="where")           then local:where($context)
	else if($context/local-name()="for")             then local:for($context)
	else if($context/local-name()="join")            then local:join($context)
	else if($context/local-name()="if")              then local:test($context)
	else if($context/local-name()="then")            then local:then($context)
	else if($context/local-name()="else")            then local:else($context)
	else if($context/local-name()="return")          then local:return($context)
	else if($context/local-name()="variable")        then local:variable($context)
	else if($context/local-name()="sequence")        then local:sequence($context)
	else if($context/local-name()="source")          then local:source($context)
	else if($context/local-name()="function_call")   then local:call($context)
	else if($context/local-name()="cache")           then local:cache($context)
	else if($context/local-name()="quantified")      then local:quantified($context)
	else if($context/local-name()="xquery")          then local:xquery($context)
	else if($context/local-name()="transform")       then local:transform($context)
	else if($context/local-name()="modify")          then local:modify($context)
	else if($context/local-name()="copy")            then local:copy($context)
	else if($context/local-name()="xuf")             then local:xuf($context)
	else if($context/local-name()="window")          then local:window($context)
	else if($context/local-name()="start-condition") then local:start-condition($context)
	else if($context/local-name()="end-condition")   then local:end-condition($context)
	else ""

};

declare function local:xml_source($context as element(*, xs:untyped)) {
    <item text="{if($context/@type='xml') then '&lt;b&gt;XML Data Source&lt;/b&gt;' else '&lt;b&gt;Streaming XML Data Source&lt;/b&gt;'}" im0="xmlSource.gif" im1="xmlSource.gif" im2="xmlSource.gif">
       {
	   if(count($context/*[@type="input"])>0) then
	        <item text="&lt;b&gt;Inputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
	        {
		        for $input in $context/* where $input/@type="input"
			    return 
				<item text="${$input/@name}" im0="varRef.gif" im1="varRef.gif" im2="varRef.gif">
				</item>
	        }
	        </item>
	   else
		   ""
       }

       {
	   if(count($context/*[@type="output"])>0) then
	       <item text="&lt;b&gt;Outputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
	       {
	          for $output in $context/* where $output/@type="output"
		      return 
		          <item text="${$output/@name}" im0="var.gif" im1="var.gif" im2="var.gif">
			      </item>
	       }
	       </item>
	    else
		   ""
       }
	   {
		   if(count($context/x:xquery)>0) then 
	   			local:xquery($context/x:xquery)
		   else if(count($context/x:xuf)>0) then 
	   			local:xuf($context/x:xuf)
		   else 
		       ""
	   }
    </item>

};



declare function local:relational_source($context as element(*, xs:untyped)) {
 
   <item text="&lt;b&gt;Relational Data Source&lt;/b&gt; ({$context/@url})" im0="dbSource.gif" im1="dbSource.gif" im2="dbSource.gif">
   {
	   if(count($context/*[local-name()='constructor'])>0) then
          for $output at $pos in $context/*
		  where $output[local-name()='constructor']
		  order by $pos descending
	      return 
		  	local:renderConstructor($output)
	   else
		   ""
   }
   {
	   for $sql_statement at $pos in $context/*
	   where $sql_statement/local-name()!='constructor'
	   order by $pos descending
	   return
	       if($sql_statement/local-name()='sequence') then
              local:sequence($sql_statement)
		   else if($sql_statement/local-name()='join') then
		      local:join($sql_statement)
		   else if($sql_statement/local-name()='variable') then
		      local:variable($sql_statement)
		   else if($sql_statement/local-name()='xquery') then
		      local:xquery($sql_statement)
		   else
			   <item text="&lt;b&gt;{local:getSQLStatementDescription($sql_statement/local-name(), false())}&lt;/b&gt;" im0="statement.gif" im1="statement.gif" im2="statement.gif">
			       {
		               if(count($sql_statement/x:tables/x:table)>0) then
					       <item text="&lt;b&gt;Tables&lt;/b&gt;">
						   {
						       for $table in $sql_statement/x:tables/x:table
							   return 
							       <item text="{$table/@name}" im0="table.gif" im1="table.gif" im2="table.gif">

								   </item>
						   }
						   </item>
					   else
					       ""
				   }

	               {
		               if(count($sql_statement/*[local-name()="parameter"])>0) then
			               <item text="&lt;b&gt;Inputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
				           {
						      for $input in $sql_statement/* where $input/local-name()="parameter"
							  return 
							      <item text="param {$input/@pos} := ${$input/@name}" im0="varRef.gif" im1="varRef.gif" im2="varRef.gif"/>
						   }
						   </item>
					   else
						   ""
				   }

				   {
				       local:renderSQLOutputs($sql_statement)
				   }
				   {
					   if(count($sql_statement/x:sql)>0) then
					       <item text="&lt;b&gt;SQL:&lt;/b&gt; {string($sql_statement/x:sql)}" im0="sql.gif" im1="sql.gif" im2="sql.gif">
					       </item>
					   else
					       ""
				   }
		       </item>
	   }
   </item>

};



declare function local:source($context as element(*, xs:untyped)) {
   if($context/@type="xml" or $context/@type="streaming-xml") then local:xml_source($context)
   else if($context/@type="relational") then local:relational_source($context)
   else ""
};


declare function local:call($context as element(*, xs:untyped)) {
    <item text = "{$context/@name}" im0="call.gif" im1="call.gif" im2="call.gif">
    {
        if(count($context/*[local-name()="parameter"])>0) then
            <item text="&lt;b&gt;Inputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
	        {
		      for $input in $context/* where $input/local-name()="parameter"
			  return 
			  	if ( $input/@name )
			  	then
			      <item text="param {$input/@pos} := ${$input/@name}" im0="varRef.gif" im1="varRef.gif" im2="varRef.gif"/>
			    else
			      <item text="param {$input/@pos} := {$input/@value}" im0="varRef.gif" im1="varRef.gif" im2="varRef.gif"/>
		    }
			</item>
		   else
			 ""
    }
	{ 
	   for $operation in $context/* order by $operation/position() descending
	   return local:switch($operation)
	}
	</item>
};


declare function local:flwor($context as element(*, xs:untyped)) {
	
	    <item text = "&lt;b&gt;flwor&lt;/b&gt;" im0="flwor.gif" im1="flwor.gif" im2="flwor.gif">
		{ 
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
		}
		</item>
};


declare function local:variable($context as element(*, xs:untyped)) {
	if($context/@type = "input") then 
	    <item text = "${$context/@name}" im0="varRef.gif" im1="varRef.gif" im2="varRef.gif">
		{ 
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
		}
		</item>
	 else if($context/@type = "positional") then
		<item text = "${$context/@name} := position()" im0="var.gif" im1="var.gif" im2="var.gif"/>
	 else if($context/@copy = "true") then
		<item text = "{$context/text()}" im0="var.gif" im1="var.gif" im2="var.gif"/>
     else
	    <item text = "${$context/@name}" im0="var.gif" im1="var.gif" im2="var.gif">
		{ 
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
		}
		</item>
};

declare function local:cache($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;cache&lt;/b&gt;" im0="cache.gif" im1="cache.gif" im2="cache.gif">
	{ 
	   for $operation in $context/* order by $operation/position() descending
	   return local:switch($operation)
	}
	</item>
};

declare function local:quantified($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;{$context/@type}&lt;/b&gt; satisfies" im0="quantified.gif" im1="quantified.gif" im2="quantified.gif">
	{ 
      (
	   if(count($context/x:satisfies/*)>0) then
        <item text="&lt;b&gt;Inputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
        {
			for $input at $pos in $context/x:satisfies/* order by $pos descending
			return local:switch($input)
        }
        </item>
	   else
		   ""
	   ),
	   for $operation at $pos in $context/*
	   where $operation/local-name() != "satisfies"
	   order by $pos descending
	   return local:switch($operation)
	}
	</item>
};

declare function local:sequence($context as element(*, xs:untyped)) {
	
	    <item text="&lt;b&gt;sequence&lt;/b&gt;" im0="sequence.gif" im1="sequence.gif" im2="sequence.gif">
		{
	    for $sql_statement in $context/* where $sql_statement/local-name()="select" or $sql_statement/local-name()="insert_temp"
	    return
	       if($sql_statement/local-name()='join') then
		      local:join($sql_statement)
		   else
			   <item text="&lt;b&gt;{local:getSQLStatementDescription($sql_statement/local-name(), false())}&lt;/b&gt;" im0="statement.gif" im1="statement.gif" im2="statement.gif">
			       {
		               if(count($sql_statement/x:tables/x:table)>0) then
					       <item text="&lt;b&gt;Tables&lt;/b&gt;">
						   {
						       for $table in $sql_statement/x:tables/x:table
							   return 
							       <item text="{$table/@name}" im0="table.gif" im1="table.gif" im2="table.gif">

								   </item>
						   }
						   </item>
					   else
					       ""
				   }

	               {
		               if(count($sql_statement/*[local-name()="parameter"])>0) then
			               <item text="&lt;b&gt;Inputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
				           {
						      for $input in $sql_statement/* where $input/local-name()="parameter"
							  return 
							      <item text="param {$input/@pos} := ${$input/@name}" im0="varRef.gif" im1="varRef.gif" im2="varRef.gif"/>
						   }
						   </item>
					   else
						   ""
				   }

				   {
					   if(count($sql_statement/*[@type="output" and not(local-name()="constructor")])>0) then
					       <item text="&lt;b&gt;Outputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
					       {
					          for $output in $sql_statement/* where $output/@type="output" and not($output/local-name()="constructor")
						      return 
						          <item text="${$output/@name} := {$output/text()}" im0="var.gif" im1="var.gif" im2="var.gif"/>
					       }
					       </item>
					   else
						   ""
				   }
				   { 
				       if(count($sql_statement/x:sql)>0) then
					       <item text="&lt;b&gt;SQL:&lt;/b&gt; {string($sql_statement/x:sql)}" im0="sql.gif" im1="sql.gif" im2="sql.gif">
					       </item>
				       else
					       ""
				   }
				   {
				   	   for $constructor in $sql_statement/x:constructor
					   	return local:renderConstructor($constructor)
				   }
		       </item>
	    }
		{ 
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
		}
		</item>
};




declare function local:let($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;let&lt;/b&gt;" im0="let.gif" im1="let.gif" im2="let.gif">
	{
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
	}
	</item>
};

declare function local:where($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;where&lt;/b&gt;" im0="where.gif" im1="where.gif" im2="where.gif">
	{
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
	}
	</item>
};

declare function local:order_by($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;order by&lt;/b&gt; { replace(replace(replace(string($context/x:clause),'&amp;','&amp;amp;'),'<','&amp;lt;'),'order by','')}" im0="order.gif" im1="order.gif" im2="order.gif">
	{
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
	}
	</item>
};

declare function local:test($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;if&lt;/b&gt;" im0="if.gif" im1="if.gif" im2="if.gif">
	{
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
	}
	</item>
};

declare function local:then($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;then&lt;/b&gt;" im0="then.gif" im1="then.gif" im2="then.gif">
	{
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
	}
	</item>
};

declare function local:else($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;else&lt;/b&gt;" im0="else.gif" im1="else.gif" im2="else.gif">
	{
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
	}
	</item>
};


declare function local:for($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;for&lt;/b&gt;" im0="for.gif" im1="for.gif" im2="for.gif">
	{
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
	}
	</item>
};

declare function local:join($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;join&lt;/b&gt; ({$context/@type})" im0="join.gif" im1="join.gif" im2="join.gif">
	{ 
	    for $operation in $context/* order by $operation/position() descending
		return local:switch($operation)
	}
	{
	    for $sql_statement in $context/* where $sql_statement/local-name()="select" or $sql_statement/local-name()="insert_temp"
	    return
	       if($sql_statement/local-name()='join') then
		      local:join($sql_statement)
		   else
			   <item text="&lt;b&gt;{local:getSQLStatementDescription($sql_statement/local-name(), false())}&lt;/b&gt;" im0="statement.gif" im1="statement.gif" im2="statement.gif">
			       {
		               if(count($sql_statement/x:tables/x:table)>0) then
					       <item text="&lt;b&gt;Tables&lt;/b&gt;">
						   {
						       for $table in $sql_statement/x:tables/x:table
							   return 
							       <item text="{$table/@name}" im0="table.gif" im1="table.gif" im2="table.gif">

								   </item>
						   }
						   </item>
					   else
					       ""
				   }

	               {
		               if(count($sql_statement/*[local-name()="parameter"])>0) then
			               <item text="&lt;b&gt;Inputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
				           {
						      for $input in $sql_statement/* where $input/local-name()="parameter"
							  return 
							      <item text="param {$input/@pos} := ${$input/@name}" im0="varRef.gif" im1="varRef.gif" im2="varRef.gif"/>
						   }
						   </item>
					   else
						   ""
				   }

				   {
					   local:renderSQLOutputs($sql_statement)
				   }
				   {
					   if(count($sql_statement/x:sql)>0) then
					       <item text="&lt;b&gt;SQL:&lt;/b&gt; {string($sql_statement/x:sql)}" im0="sql.gif" im1="sql.gif" im2="sql.gif">
					       </item>
					   else
					       ""
				   }
		       </item>
	}
	</item>
};


declare function local:start-condition($context as element(*, xs:untyped)) {
  <item text="&lt;b&gt;start&lt;/b&gt;" im0="tumblingWindowStart.gif" im1="tumblingWindowStart.gif" im2="tumblingWindowStart.gif">
	{
		   if(count($context/*[local-name()="variable"])>0) then
		       <item text="&lt;b&gt;Outputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
		       {
		          for $output in $context/* where $output/local-name()="variable"
			        return 
			          <item text="${$output/@name}" im0="var.gif" im1="var.gif" im2="var.gif"/>
		       }
		       </item>
		   else
			   ""
       ,
       local:switch($context/*[last()])
	}
	</item>
};


declare function local:end-condition($context as element(*, xs:untyped)) {
  <item text="&lt;b&gt;{if ($context/@only) then 'only ' else ''}end&lt;/b&gt;" im0="tumblingWindowEnd.gif" im1="tumblingWindowEnd.gif" im2="tumblingWindowEnd.gif">
	{
		   if(count($context/*[local-name()="variable"])>0) then
		       <item text="&lt;b&gt;Outputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
		       {
		          for $output in $context/* where $output/local-name()="variable"
			        return 
			          <item text="${$output/@name}" im0="var.gif" im1="var.gif" im2="var.gif"/>
		       }
		       </item>
		   else
			   ""
       ,
       local:switch($context/*[last()])
	}
	</item>
};


declare function local:window($context as element(*, xs:untyped)) {
  <item text="&lt;b&gt;{if ($context/@type eq 'tumbling') then 'tumbling ' else 'sliding '}window&lt;/b&gt;" im0="tumblingWindow.gif" im1="tumblingWindow.gif" im2="tumblingWindow.gif">
	{
     if(count($context/*[local-name()="variable" and @type="input"])>0) then
        <item text="&lt;b&gt;Inputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
        {
          for $input in $context/* where $input/local-name()="variable" and $input/@type="input"
          return 
            <item text="${$input/@name}" im0="varRef.gif" im1="varRef.gif" im2="varRef.gif"/>
        }
        </item>
     else
       ""
     ,
	   if(count($context/*[local-name()="variable" and @type="output"])>0) then
	       <item text="&lt;b&gt;Outputs&lt;/b&gt;" im0="folderClosed.gif" im1="folderOpen.gif" im2="folderClosed.gif">
	       {
	          for $output in $context/* where $output/local-name()="variable" and $output/@type="output"
		        return 
		          <item text="${$output/@name}" im0="var.gif" im1="var.gif" im2="var.gif"/>
	       }
	       </item>
	   else
		   ""
     ,
     local:start-condition($context/x:start-condition)
     ,
     if ($context/x:end-condition) then local:end-condition($context/x:end-condition) else ""
	}
	</item>
};

declare function local:xquery($context as element(*, xs:untyped)) {
   <item text="&lt;b&gt;XQuery:&lt;/b&gt; {replace(replace(string($context),'&amp;','&amp;amp;'),'<','&amp;lt;')}" im0="xquery.gif" im1="xquery.gif" im2="xquery.gif">
   </item>
};

declare function local:xuf($context as element(*, xs:untyped)) {
   <item text="&lt;b&gt;XQuery Update:&lt;/b&gt; {replace(replace(string($context),'&amp;','&amp;amp;'),'<','&amp;lt;')}" im0="xu.gif" im1="xu.gif" im2="xu.gif">
   </item>
};

declare function local:transform($context as element(*, xs:untyped)) {
	
	    <item text = "&lt;b&gt;transform&lt;/b&gt;" im0="transform.gif" im1="transform.gif" im2="transform.gif">
		{ 
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
		}
		</item>
};

declare function local:modify($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;modify&lt;/b&gt;" im0="modify.gif" im1="modify.gif" im2="modify.gif">
	{
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
	}
	</item>
};

declare function local:copy($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;copy&lt;/b&gt;" im0="copy.gif" im1="copy.gif" im2="copy.gif">
	{
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
	}
	</item>
};

declare function local:return($context as element(*, xs:untyped)) {
    <item text="&lt;b&gt;return&lt;/b&gt;" im0="return.gif" im1="return.gif" im2="return.gif">
	{
		   for $operation in $context/* order by $operation/position() descending
		   return local:switch($operation)
	}
	</item>
};


declare function local:index($context as node()*, $id as xs:integer) {
    if(count($context/*)>0) then
		let $x := local:index($context/*[1],$id+1)
		return (
		    if($context/local-name()="xmp") then 
			(
			    <xmp>
				{
					 subsequence($x, 1,  count($x)-1)
				}
				</xmp>,
				if(count($context/following-sibling::*)>0) then
				   local:index($context/following-sibling::*[1],xs:integer(number($x[count($x)]))+1)
				else
				   $x[count($x)]
			)	
			else
			(
			    <item im0="{$context/@im0}" im1="{$context/@im1}" im2="{$context/@im2}" id="{$id}">
				<itemtext>
				{
				  string($context/@text)
				}
				</itemtext>
				{
					 subsequence($x, 1,  count($x)-1)
				}
				</item>,
				if(count($context/following-sibling::*)>0) then
				   local:index($context/following-sibling::*[1],xs:integer(number($x[count($x)]))+1)
				else
				   $x[count($x)]
			)
		)
	else (
	    if($context/local-name()="xmp") then 
		(    
		    <xmp/>,
		    if(count($context/following-sibling::*)>0) then
		         local:index($context/following-sibling::*[1],$id+1)
			else
			   $id
		)
		else
		(
		    <item im0="{$context/@im0}" im1="{$context/@im1}" im2="{$context/@im2}" id="{$id}">
			   <itemtext>
			   {
			      string($context/@text)
			   }
			   </itemtext>
			</item>,
		    if(count($context/following-sibling::*)>0) then
		         local:index($context/following-sibling::*[1],$id+1)
			else
			   $id
		)
	)
};

declare function local:indexed_tree() {
	let $out :=
		local:index(
		<xmp>
		    { local:adaptorsSection($planExplainDoc/x:explain)}
	        { local:globalVariablesSection($planExplainDoc/x:explain) }
			{ local:functionsSection($planExplainDoc/x:explain) }
			{ local:mainSection($planExplainDoc/x:explain) }
	    </xmp>		,
		0)
	return subsequence($out, 1,  count($out)-1)
};


<html>
    <head>
		<base href="{local:getResouceLocation()}"/>
	</head>
	<body>
		<script  src="js/dhtmlXCommon.js"><p/></script>
		<script  src="js/dhtmlXTree.js"><p/></script>
		<script  src="js/dhtmlXTree_xw.js"><p/></script>
		<script  src="js/dhtmlXTree_start.js"><p/></script>
		<script  src="js/PlanExplain_start.js"><p/></script>
		<script  src="js/dhtmlXTree_kn.js"><p/></script>
	    <script language="JavaScript" src="menu/js/dhtmlXProtobar.js"><p/></script>
	    <script language="JavaScript" src="menu/js/dhtmlXMenuBar.js"><p/></script>
	    <script language="JavaScript" src="menu/js/dhtmlXMenuBar_cp.js"><p/></script>
		<link rel="STYLESHEET" type="text/css" href="css/dhtmlXTree.css"></link>
	    <link rel="STYLESHEET" type="text/css" href="menu/css/dhtmlXMenu.css"></link>
	    <link rel="STYLESHEET" type="text/css" href="menu/css/Context.css"></link>
		<table style="width:100%; height:100%">
		    <tr style="width:100%;height:32">
			    <td>
				    <div style="width:100%;height:32">
					    <img id="definition"     src="imgs/GoToDefinitionBtnDisabled.gif" alt="Go To Definition" onclick="definitionBtnClicked()"/>
					    <img id="reference"      src="imgs/GoToFirstReferenceBtnDisabled.gif" alt="Go To First Reference" onclick="firstReferenceBtnClicked()"/>
					    <img id="next_reference" src="imgs/GoToNextReferenceBtnDisabled.gif" alt="Go To Next Reference" onclick="nextReferenceBtnClicked()"/>
					    <img id="prev_reference" src="imgs/GoToPrevReferenceBtnDisabled.gif" alt="Go To Previous Reference" onclick="prevReferenceBtnClicked()"/>
					</div>
				</td>
			</tr>
			<tr>
			    <td>
					<div id="PlanExplain" enableTreeLines="0" enableKeyboardNavigation="true" setImagePath="imgs/" style="width:100%; height:100%; background-color:#f5f5f5;border :1px solid Silver; overflow:auto;">
					   { local:indexed_tree() }
					</div>
				</td>
			</tr>
		</table>
		<script>{"
			tree=new PlanExplainTreeFromHTML('PlanExplain');
		"} </script>
	</body>
</html>
